package com.hikapro.backpack.presenter.adapters.helper.items;

import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Canvas;
import android.graphics.drawable.Drawable;
import android.support.v4.content.ContextCompat;
import android.support.v7.widget.RecyclerView;
import android.view.View;

/**
 * Created by tariel on 19/07/16.
 */
public class DividerDecoration2 extends RecyclerView.ItemDecoration {

    private static final int[] ATTRS = new int[]{android.R.attr.listDivider};
    private int paddingX;

    private Drawable mDivider;

    /**
     * Default divider will be used
     */
    public DividerDecoration2(Context context) {
        final TypedArray styledAttributes = context.obtainStyledAttributes(ATTRS);
        mDivider = styledAttributes.getDrawable(0);
        styledAttributes.recycle();
    }

    /**
     * Custom divider will be used
     */
    public DividerDecoration2(Context context, int resId) {
        mDivider = ContextCompat.getDrawable(context, resId);
    }

    public DividerDecoration2(Context context, int resId, int paddingX) {
        mDivider = ContextCompat.getDrawable(context, resId);
        this.paddingX = paddingX;
    }

    @Override
    public void onDraw(Canvas c, RecyclerView parent, RecyclerView.State state) {
        int left = parent.getPaddingLeft();
        int right = parent.getWidth() - parent.getPaddingRight();

        int childCount = parent.getChildCount();
        for (int i = 0; i < childCount; i++) {
            View child = parent.getChildAt(i);

            RecyclerView.LayoutParams params = (RecyclerView.LayoutParams) child.getLayoutParams();

            int top = child.getBottom() + params.bottomMargin;
            int bottom = top + mDivider.getIntrinsicHeight();

            mDivider.setBounds(left + paddingX, top, right - paddingX, bottom);
            mDivider.draw(c);
        }
    }
}
